@@ -1430,7 +1430,7 @@ _gnutls_proc_cert_cert_req(gnutls_session_t session, uint8_t * data,
 	/* We should reply with a certificate message, 
 	 * even if we have no certificate to send.
 	 */
-	session->key.crt_requested = 1;
+	session->internals.crt_requested = 1;
 
 	/* now we ask the user to tell which one
 	 * he wants to use.
