@@ -43,6 +43,8 @@
 # include <signal.h>
 #endif
 
+#include <assert.h>
+
 /* Gnulib portability files. */
 #include <read-file.h>
 
@@ -566,6 +568,18 @@ generate_certificate(gnutls_privkey_t * ret_key,
 			}
 		}
 
+		result = get_email_protection_status();
+		if (result) {
+			result =
+			    gnutls_x509_crt_set_key_purpose_oid
+			    (crt, GNUTLS_KP_EMAIL_PROTECTION, 0);
+			if (result < 0) {
+				fprintf(stderr, "key_kp: %s\n",
+					gnutls_strerror(result));
+				exit(1);
+			}
+		}
+
 		if (ca_status) {
 			result = get_cert_sign_status();
 			if (result)
@@ -1053,7 +1067,7 @@ static void cmd_parser(int argc, char **argv)
 	if (HAVE_OPT(OUTFILE)) {
 		outfile = safe_open_rw(OPT_ARG(OUTFILE), privkey_op);
 		if (outfile == NULL) {
-			fprintf(stderr, "%s", OPT_ARG(OUTFILE));
+			fprintf(stderr, "Cannot open %s for writing\n", OPT_ARG(OUTFILE));
 			exit(1);
 		}
 	} else
@@ -1067,7 +1081,7 @@ static void cmd_parser(int argc, char **argv)
 
 		infile = fopen(OPT_ARG(INFILE), "rb");
 		if (infile == NULL) {
-			fprintf(stderr, "%s", OPT_ARG(INFILE));
+			fprintf(stderr, "Cannot open %s for reading\n", OPT_ARG(INFILE));
 			exit(1);
 		}
 	} else
@@ -1617,17 +1631,6 @@ void pgp_privkey_info(void)
 
 			fprintf(outfile, "Fingerprint: %s\n",
 				raw_to_string(lbuffer, size));
-
-			ret =
-			    gnutls_random_art(GNUTLS_RANDOM_ART_OPENSSH,
-					      cprint, bits, lbuffer, size,
-					      &art);
-			if (ret >= 0) {
-				fprintf(outfile,
-					"Fingerprint's random art:\n%s\n\n",
-					art.data);
-				gnutls_free(art.data);
-			}
 		}
 	}
 
@@ -1903,7 +1906,10 @@ void privkey_info(common_info_st * cinfo)
 	if (ret == GNUTLS_E_DECRYPTION_FAILED) {
 		fprintf(stderr, "Encrypted structure detected...\n");
 
-		pkcs8_info_int(&pem, incert_format, 1, outfile, "");
+		if (outcert_format == GNUTLS_X509_FMT_DER)
+			pkcs8_info_int(&pem, incert_format, 1, stderr, "");
+		else
+			pkcs8_info_int(&pem, incert_format, 1, outfile, "");
 
 		pass = get_password(cinfo, &flags, 0);
 
@@ -2074,6 +2080,18 @@ void generate_request(common_info_st * cinfo)
 			}
 		}
 
+		ret = get_email_protection_status();
+		if (ret) {
+			ret =
+			    gnutls_x509_crq_set_key_purpose_oid
+			    (crq, GNUTLS_KP_EMAIL_PROTECTION, 0);
+			if (ret < 0) {
+				fprintf(stderr, "key_kp: %s\n",
+					gnutls_strerror(ret));
+				exit(1);
+			}
+		}
+
 		ret = get_ipsec_ike_status();
 		if (ret) {
 			ret = gnutls_x509_crq_set_key_purpose_oid
@@ -2913,8 +2931,10 @@ void verify_pkcs7(common_info_st * cinfo, const char *purpose, unsigned display_
 					ret = GNUTLS_E_CONSTRAINT_ERROR;
 			}
 
-		} else
+		} else {
+			assert(tl != NULL);
 			ret = gnutls_pkcs7_verify(pkcs7, tl, vdata, vdata_size, i, detached.data!=NULL?&detached:NULL, flags);
+		}
 		if (ret < 0) {
 			fprintf(stderr, "\tSignature status: verification failed: %s\n", gnutls_strerror(ret));
 			ecode = 1;
@@ -3378,7 +3398,10 @@ static void print_bag_data(gnutls_pkcs12_bag_t bag)
 		}
 
 		if (type == GNUTLS_BAG_PKCS8_ENCRYPTED_KEY) {
-			pkcs8_info_int(&cdata, GNUTLS_X509_FMT_DER, 1, outfile, "\t");
+			if (outcert_format == GNUTLS_X509_FMT_DER)
+				pkcs8_info_int(&cdata, GNUTLS_X509_FMT_DER, 1, stderr, "\t");
+			else
+				pkcs8_info_int(&cdata, GNUTLS_X509_FMT_DER, 1, outfile, "\t");
 		}
 
 		name = NULL;
@@ -3647,8 +3670,11 @@ void pkcs8_info_int(gnutls_datum_t *data, unsigned format,
 	if (ret == GNUTLS_E_UNKNOWN_CIPHER_TYPE) {
 		fprintf(out, "PKCS #8 information:\n");
 		fprintf(out, "\tSchema: unsupported (%s)\n", oid);
-		gnutls_free(oid);
-		return;
+		goto cleanup;
+	} else if (ret == GNUTLS_E_INVALID_REQUEST) {
+		fprintf(out, "PKCS #8 information:\n");
+		fprintf(out, "\tSchema: unencrypted key\n");
+		goto cleanup;
 	}
 
 	if (ret < 0) {
@@ -3658,16 +3684,16 @@ void pkcs8_info_int(gnutls_datum_t *data, unsigned format,
 			gnutls_strerror(ret));
 		exit(1);
 	}
-	gnutls_free(oid);
 
 	fprintf(out, "%sPKCS #8 information:\n", tab);
 	fprintf(out, "%s\tCipher: %s\n", tab, gnutls_cipher_get_name(cipher));
 
 	str = gnutls_pkcs_schema_get_name(schema);
 	if (str != NULL) {
-		fprintf(out, "%s\tSchema: %s (%s)\n", tab, str, gnutls_pkcs_schema_get_oid(schema));
+		fprintf(out, "%s\tSchema: %s (%s)\n", tab, str,  gnutls_pkcs_schema_get_oid(schema));
 	}
 
+
 	bin.data = salt;
 	bin.size = salt_size;
 	ret = gnutls_hex_encode(&bin, hex, &hex_size);
@@ -3680,6 +3706,9 @@ void pkcs8_info_int(gnutls_datum_t *data, unsigned format,
 	fprintf(out, "%s\tSalt: %s\n", tab, hex);
 	fprintf(out, "%s\tSalt size: %u\n", tab, salt_size);
 	fprintf(out, "%s\tIteration count: %u\n\n", tab, iter_count);
+
+ cleanup:
+	gnutls_free(oid);
 }
 
 void pkcs8_info(void)
@@ -3789,6 +3818,7 @@ void smime_to_pkcs7(void)
 	}
 	while (strcmp(lineptr, "\r\n") != 0 && strcmp(lineptr, "\n") != 0);
 
+	/* skip newlines */
 	do {
 		len = getline(&lineptr, &linesize, infile);
 		if (len == -1) {
@@ -3797,7 +3827,7 @@ void smime_to_pkcs7(void)
 			exit(1);
 		}
 	}
-	while (strcmp(lineptr, "\r\n") == 0 && strcmp(lineptr, "\n") == 0);
+	while (strcmp(lineptr, "\r\n") == 0 || strcmp(lineptr, "\n") == 0);
 
 	fprintf(outfile, "%s", "-----BEGIN PKCS7-----\n");
 
