@@ -20,10 +20,11 @@ int main(int argc, char **argv)
 	exit(77);
 }
 #else
+# include <unistd.h>
 
 static void handle_sigill(int sig)
 {
-	exit(0);
+	_exit(0);
 }
 
 static void tls_log_func(int level, const char *str)
